<?php
/**
 * XO Security log list table class.
 *
 * @package xo-security
 * @since 1.0.0
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

class XO_Login_Log_List_Table extends WP_List_Table {
	public $default_status = '';

	public function __construct() {
		parent::__construct( array(
			'singular' => 'loginlog',
			'plural'   => 'loginlogs',
			'ajax'     => false,
		) );

		wp_get_current_user();
	}

	public function column_default( $item, $column_name ) {
		return $item[$column_name];
	}

	public function column_success( $item ) {
		return $item['success'] ? __( 'Success', 'xo-security' ) : __( 'Failure', 'xo-security' );
	}

	public function column_cb( $item ) {
		/**
		 * Filters whether to show the bulk edit checkbox for login log.
		 *
		 * @since 3.4.2
		 *
		 * @param bool $show Whether to show the checkbox.
		 */
		if ( apply_filters( 'xo_security_loginlog_checkbox', true ) ) {
			echo sprintf( '<input type="checkbox" name="%1$s[]" value="%2$s" />', esc_attr( $this->_args['singular'] ), esc_attr( $item['id'] ) );
		}
	}

	public function column_type( $item ) {
		$s = '';
		if ( $item['type'] == XO_Security::LOGIN_TYPE_LOGIN_PAGE ) {
			$s = __( 'Login page', 'xo-security' );
		} else if ( $item['type'] == XO_Security::LOGIN_TYPE_XMLRPC ) {
			$s = __( 'XMLRPC', 'xo-security' );
		}
		return $s;
	}

	public function get_columns() {
		global $current_user;

		$columns = array(
			'cb'         => '<input type="checkbox" />',
			'login_time' => __( 'Date', 'xo-security' ),
			'success'    => __( 'Result', 'xo-security' ),
			'ip_address' => __( 'IP address', 'xo-security' ),
			'type'       => __( 'Type', 'xo-security' ),
			'lang'       => __( 'Language', 'xo-security' ),
			'user_agent' => __( 'User Agent', 'xo-security' ),
		);
		if ( $current_user->has_cap( 'administrator' ) ) {
			$columns['user_name'] = __( 'Username', 'xo-security' );
			$columns['failed_password'] = __( 'Password', 'xo-security' );
		}
		return $columns;
	}

	protected function get_sortable_columns() {
		$sortable_columns = array(
			'login_time' => array( 'logintime', true ),
			'success'    => array( 'success', false ),
			'ip_address' => array( 'ipaddress', false ),
			'type'       => array( 'type', false ),
			'lang'       => array( 'lang', false ),
			'user_name'  => array( 'username', false ),
		);
		return $sortable_columns;
	}

	protected function get_bulk_actions() {
		$actions = array( 'delete' => __( 'Delete', 'xo-security' ) );
		return $actions;
	}

	protected function status_dropdown() {
		$status = isset( $_GET['status'] ) ? $_GET['status'] : $this->default_status;
		echo '<label for="filter-by-status" class="screen-reader-text">' . esc_html__( 'Filter by results', 'xo-security' ) . '</label>';
		echo '<select name="status" id="filter-by-status">';
		echo '<option' . selected( $status, '', false ) . ' value="">' . esc_html__( 'All results', 'xo-security' ) . '</option>';
		printf( "<option %s value='%s'>%s</option>\n", selected( $status, '0', false ), '0', esc_html__( 'Failure', 'xo-security' ) );
		printf( "<option %s value='%s'>%s</option>\n", selected( $status, '1', false ), '1', esc_html__( 'Success', 'xo-security' ) );
		echo '</select>' . "\n";
	}

	protected function users_dropdown() {
		global $current_user;

		if ( $current_user->has_cap( 'administrator' ) ) {
			$users = get_users();
			$user_login = isset( $_GET['u'] ) ? $_GET['u'] : '';
			echo '<label for="filter-by-user" class="screen-reader-text">' . esc_html__( 'Filter by user', 'xo-security' ). '</label>';
			echo '<select name="u" id="filter-by-user">';
			echo '<option' . selected( $user_login, '', false ) . ' value="">' . esc_html__( 'All users', 'xo-security' ) . '</option>';
			foreach ( $users as $user ) {
				printf( "<option %s value='%s'>%s</option>\n", selected( $user_login, $user->user_login, false ), esc_html( $user->user_login ), esc_html( $user->user_login ) );
			}
			echo '</select>' . "\n";
		} else {
			echo '<label for="filter-by-user" class="screen-reader-text">' . esc_html__( 'Filter by user', 'xo-security' ). '</label>';
			echo '<select name="u" id="filter-by-user">';
			printf( '<option selected="selected" value="%s">%s</option>', esc_html( $current_user->user_login ), esc_html( $current_user->user_login ) );
			echo '</select>' . "\n";
		}
	}

	protected function extra_tablenav( $which ) {
		echo '<div class="alignleft actions">';
		if ( 'top' === $which && ! is_singular() ) {
			$this->status_dropdown();
			$this->users_dropdown();
			submit_button( __( 'Filter', 'xo-security' ), 'button', 'filter_action', false, array( 'id' => 'post-query-submit' ) );
		}
		echo '</div>';
	}

	public function process_bulk_action() {
		global $wpdb, $current_user;

		if ( 'delete' === $this->current_action() ) {
			$table = $wpdb->prefix . 'xo_security_loginlog';
			if ( isset( $_GET['loginlog'] ) ) {
				$ids = $_GET['loginlog'];
				if ( ! is_array( $ids ) ) {
					$ids = array( $ids );
				}
				$ids = implode( ',', array_map( 'absint', $ids ) );
				if ( ! empty( $ids ) ) {
					if ( $current_user->has_cap( 'administrator' ) ) {
						$wpdb->query( "DELETE FROM `$table` WHERE id IN ($ids);" );
					} else {
						$wpdb->query( $wpdb->prepare( "DELETE FROM `$table` WHERE id IN ($ids) AND user_name = %s;", $current_user->user_login ) );
					}
				}
			}
		}
	}

	public function prepare_items() {
		global $wpdb, $current_user;

		$table = $wpdb->prefix . 'xo_security_loginlog';
		$per_page = $this->get_items_per_page( 'login_log_per_page', 100 );
		$status = isset( $_GET['status'] ) ? sanitize_text_field( $_GET['status'] ) : $this->default_status;

		if ( $current_user->has_cap( 'administrator' ) ) {
			$user_login = isset( $_GET['u'] ) ? sanitize_text_field( $_GET['u'] ) : '';
		} else {
			$user_login = $current_user->user_login;
		}

		list( $columns, $hidden ) = $this->get_column_info();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array( $columns, $hidden, $sortable );

		$this->process_bulk_action();

		// WHERE
		$wheres = array();
		if ( '' !== $status ) {
			$wheres[] = $wpdb->prepare( 'success=%d', $status );
		}
		if ( '' !== $user_login ) {
			$wheres[] = $wpdb->prepare( 'user_name=%s', $user_login );
		}
		$where = ( count( $wheres ) ) ? ' WHERE ' . implode( ' AND ', $wheres ) : '';

		// Get all data records
		$total_items = intval( $wpdb->get_var( "SELECT count(*) FROM `{$table}`{$where}" ) );

		// Get one page of data
		$orderby_values = array( 'success', 'logintime', 'ipaddress', 'type', 'lang', 'username' );
		$order_values = array( 'asc', 'desc' );

		$orderby = ( ! empty( $_REQUEST['orderby'] ) ) ? ( in_array( $_REQUEST['orderby'], $orderby_values ) ? $_REQUEST['orderby'] : 'logintime' ) : 'logintime';
		$order = ( ! empty( $_REQUEST['order'] ) ) ? ( in_array( $_REQUEST['order'], $order_values ) ? $_REQUEST['order'] : 'desc' ) : 'desc';
		$current_page = $this->get_pagenum();
		$start = ( $current_page - 1 ) * $per_page;
		$rows = $wpdb->get_results( $wpdb->prepare(
			"SELECT `id`,`success`,`login_time` AS `logintime`,`ip_address` AS `ipaddress`,`login_type` AS `type`,`lang`,`user_agent` AS `useragent`,`user_name` AS `username`,`failed_password` AS `failedpassword`" .
			"FROM {$table}{$where} ORDER BY `{$orderby}` {$order} LIMIT %d, %d;", $start, $per_page
		), 'ARRAY_A' );
		foreach( $rows as $row ) {
			$item = array(
				'id'              => esc_html( $row['id'] ),
				'success'         => esc_html( $row['success'] ),
				'login_time'      => esc_html( $row['logintime'] ),
				'ip_address'      => esc_html( $row['ipaddress'] ),
				'type'            => esc_html( $row['type'] ),
				'lang'            => esc_html( $row['lang'] ),
				'user_agent'      => esc_html( $row['useragent'] ),
				'user_name'       => esc_html( $row['username'] ),
				'failed_password' => esc_html( $row['failedpassword'] ),
			);
			$this->items[] = $item;
		}

		$this->set_pagination_args( array( 'total_items' => $total_items, 'per_page' => $per_page, 'total_pages' => ceil( $total_items / $per_page ) ) );
	}

	protected function get_primary_column_name() {
		return 'login_time';
	}
}
