<?php
/**
 * XO Security main class.
 *
 * @package xo-security
 * @since 1.0.0
 */

class XO_Security {
	const LOGIN_TYPE_UNKNOWN    = 0;
	const LOGIN_TYPE_LOGIN_PAGE = 1;
	const LOGIN_TYPE_XMLRPC     = 2;

	const MAX_LOGIN_LOG_RECORDS = 100000;

	public $options;

	private $hash_version;

	/**
	 * Construction.
	 *
	 * @since 1.0.0
	 *
	 * @global wpdb $wpdb WordPress database abstraction object.
	 */
	public function __construct() {
		load_plugin_textdomain( 'xo-security' );

		if ( is_admin() ) {
			require_once __DIR__ . '/class-xo-security-admin.php';
			new XO_Security_Admin( $this );
		}

		$this->options = get_option( 'xo_security_options' );
		if ( false === $this->options ) {
			$this->activation();
			$this->options = get_option( 'xo_security_options' );
		} else {
			if ( ! isset( $this->options['plugin_version'] ) || version_compare( $this->options['plugin_version'], XO_SECURITY_VERSION, '<' ) ) {
				$this->activation();
				$this->options = get_option( 'xo_security_options' );
			}
		}

		$login_captcha = isset( $this->options['login_captcha'] ) ? $this->options['login_captcha'] : '';
		$comment_captcha = isset( $this->options['comment_captcha'] ) ? $this->options['comment_captcha'] : '';
		if ( ! is_admin() && ( $login_captcha != '' || $comment_captcha != '' ) ) {
			$this->session_start();
		}

		add_action( 'plugins_loaded', array( $this, 'setup' ), 99999 );
	}

	/**
	 * Plugin activation.
	 *
	 * @since 1.0.0
	 *
	 * @global wpdb $wpdb WordPress database abstraction object.
	 *
	 * @return void
	 */
	public static function activation() {
		global $wpdb;

		$table = $wpdb->prefix . 'xo_security_loginlog';

		if ( $wpdb->get_var( $wpdb->prepare( "SHOW TABLES LIKE %s;", $table ) ) !== $table ) {
			$charset_collate = $wpdb->get_charset_collate();
			$wpdb->query(
				"CREATE TABLE {$table} (" .
				"`id` bigint(20) unsigned NOT NULL AUTO_INCREMENT," .
				"`success` boolean NOT NULL DEFAULT '0'," .
				"`login_time` datetime NOT NULL DEFAULT '0000-00-00 00:00:00'," .
				"`ip_address` varchar(46) DEFAULT NULL," .
				"`login_type` tinyint(1) DEFAULT '0'," .
				"`lang` varchar(5) DEFAULT NULL," .
				"`user_agent` varchar(255) DEFAULT NULL," .
				"`user_name` varchar(255) DEFAULT NULL," .
				"`failed_password` varchar(255) DEFAULT NULL," .
				"PRIMARY KEY (`id`)," .
				"KEY success(success)," .
				"KEY login_time(login_time)," .
				"KEY ip_address(ip_address)," .
				"KEY user_name(user_name)" .
				") {$charset_collate};"
			);
		} else {
			// For 2.4.1 or earlier
			$columns = $wpdb->get_results( "SHOW COLUMNS FROM {$table} WHERE Field='login_type';" );
			if ( ! $columns ) {
				$wpdb->query( "ALTER TABLE {$table} CHANGE `ip_address` `ip_address` VARCHAR(46) DEFAULT NULL;" );
				$wpdb->query( "ALTER TABLE {$table} ADD `login_type` tinyint(1) DEFAULT '0';" );
			}

			// For 3.3.0 or earlier
			$indexes = $wpdb->get_results( "SHOW INDEXES FROM {$table};" );
			$keys = array_column( (array) $indexes, 'Key_name' );
			if ( false === array_search( 'success', $keys ) ) {
				$wpdb->query( "ALTER TABLE {$table} ALTER success SET DEFAULT '0';" );
				$wpdb->query( "ALTER TABLE {$table} ADD INDEX success(success);" );
			}
			if ( false === array_search( 'user_name', $keys ) ) {
				$wpdb->query( "ALTER TABLE {$table} ADD INDEX user_name(user_name);" );
			}

			$wpdb->query( "OPTIMIZE TABLE {$table}" );
		}

		$options = get_option( 'xo_security_options' );
		if ( $options ) {
			$options['plugin_version'] = XO_SECURITY_VERSION;
		} else {
			$options = self::get_default_options();
		}
		update_option( 'xo_security_options', $options );
	}

	/**
	 * Plugin uninstall by site.
	 *
	 * @since 3.1.0
	 *
	 * @global wpdb $wpdb WordPress database abstraction object.
	 *
	 * @return void
	 */
	public static function uninstall_site() {
		global $wpdb;

		$table = $wpdb->prefix . 'xo_security_loginlog';
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', $table ) ) !== null ) {
			$wpdb->query( "DROP TABLE {$table};" );
		}

		delete_option( 'xo_security_options' );
	}

	/**
	 * Plugin uninstall.
	 *
	 * @since 1.0.0
	 *
	 * @global wpdb $wpdb WordPress database abstraction object.
	 *
	 * @return void
	 */
	public static function uninstall() {
		global $wpdb;

		if ( ! is_multisite() ) {
			XO_Security::uninstall_site();
		} else {
			$current_blog_id = get_current_blog_id();
			$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
			foreach ( $blog_ids as $blog_id ) {
				switch_to_blog( $blog_id );
				XO_Security::uninstall_site();
			}
			switch_to_blog( $current_blog_id );
		}
	}

	/**
	 * Get default options data.
	 * 
	 * @since 1.7.0
	 * 
	 * @return array default options data.
	 */
	public static function get_default_options() {
		$default_options = array(
			'plugin_version'       => XO_SECURITY_VERSION,
			'interval'             => 0,
			'limit_count'          => 4,
			'blocked_tarpit'       => 10,
			'failed_tarpit'        => 1,
			'login_page'           => false,
			'login_page_name'      => '',
			'ms_common_login_page' => true,
			'password_reset'       => true,
			'login_site_link'      => true,
			'comment_spam'         => false,
			'comment_spam_email'   => false,
			'comment_spam_action'  => 'block', // 'block', 'spam' or 'trash'
			'comment_bat'          => false,
			'xmlrpc'               => false,
			'pingback'             => false,
			'rest'                 => false,
			'rest_rename'          => false,
			'rest_name'            => '',
			'author_archive'       => false,
			'edit_author_slug'     => false,
			'comment_author_class' => false,
			'oembed_author'        => false,
			'error_login_message'  => false,
			'login_id_type'        => '', // ''(username or email), 'username' or 'email'
			'ip_mode'              => '', // ''(Auto), 'http_x_forward_for_1', 'http_x_forward_for_2', 'http_x_forward_for_3', 'remote_addr', 'http_x_real_ip' or 'http_client_ip'
			'auto_truncate'        => 365,
			'disable_feed'         => false,
			'log_default_status'   => '', // '': All, '0': Failure or '1': Success
		);
		return $default_options;
	}

	/**
	 * Plugin setup.
	 * 
	 * @since 1.0.0
	 * 
	 * @global string $wp_version The WordPress version string.
	 */
	public function setup() {
		global $wp_version;

		if ( ! is_main_site() ) {
			switch_to_blog( 1 );
			$options = get_option( 'xo_security_options' );
			restore_current_blog();

			$this->options['ms_common_login_page'] = isset( $options['ms_common_login_page'] ) ? $options['ms_common_login_page'] : false;
			if ( $this->options['ms_common_login_page'] ) {
				$this->options['login_page'] = isset( $options['login_page'] ) ? $options['login_page'] : false;
				$this->options['login_page_name'] = isset( $options['login_page_name'] ) ? $options['login_page_name'] : '';
			}
		}

		add_filter( 'authenticate', array( $this, 'authenticate' ), 100, 3 );
		add_action( 'wp_login', array( $this, 'handler_wp_login' ), 1, 2 );
		add_action( 'xmlrpc_call', array( $this, 'handler_xmlrpc_call' ), 10, 1 );
		add_filter( 'xmlrpc_login_error', array( $this, 'xmlrpc_login_error_message' ) );
		add_action( 'login_init', array( $this, 'login_init' ) );
		add_filter( 'login_errors', array( $this, 'login_error_message' ) );
		add_filter( 'shake_error_codes', array( $this, 'login_failure_shake' ) );
		add_filter( 'wp_authenticate_user', array( $this, 'authenticate_user' ), 99999, 2 );

		if ( isset( $this->options['login_page'] ) && $this->options['login_page'] ) {
			add_filter( 'site_url', array( $this, 'site_url' ), 10, 4 );
			add_filter( 'network_site_url', array( $this, 'network_site_url' ), 10, 3 );
			add_filter( 'wp_redirect', array( $this, 'login_page_wp_redirect' ), 10, 2 );
			add_filter( 'template_redirect', array( $this, 'recreate_login_file' ) );
		}

		if ( isset( $this->options['author_archive'] ) && $this->options['author_archive'] ) {
			add_filter( 'author_rewrite_rules', array( $this, 'author_rewrite_rules' ) );
			add_action( 'init', array( $this, 'author_rewrite' ) );
			add_filter( 'wp_sitemaps_add_provider', array( $this, 'remove_author_sitemap_provider' ), 10, 2 );
		}

		if ( isset( $this->options['comment_author_class'] ) && $this->options['comment_author_class'] ) {
			add_filter( 'comment_class', array( $this, 'remove_comment_author_class' ) );
		}

		if ( isset( $this->options['oembed_author'] ) && $this->options['oembed_author'] ) {
			add_filter( 'oembed_response_data', array( $this, 'filter_oembed_response_data' ) );
		}

		if ( isset( $this->options['xmlrpc'] ) && $this->options['xmlrpc'] ) {
			add_filter( 'xmlrpc_enabled', array( $this, 'xmlrpc_enabled' ) );
		}

		if ( isset( $this->options['pingback'] ) && $this->options['pingback'] ) {
			add_filter( 'xmlrpc_methods', array( $this, 'remove_pingback' ) );
			add_filter( 'wp_headers', array( $this, 'remove_x_pingback' ) );
		}

		if ( isset( $this->options['rest'] ) && $this->options['rest'] ) {
			if ( version_compare( $wp_version, '4.7' ) >= 0 ) {
				add_filter( 'rest_endpoints', array( $this, 'rest_endpoints' ), 10, 1 );
			} else {
				$this->disable_rest();
			}
		}

		if ( isset( $this->options['rest_rename'] ) && $this->options['rest_rename'] ) {
			if ( has_action( 'wp_head', 'rest_output_link_wp_head' ) ) {
				remove_action( 'wp_head', 'rest_output_link_wp_head', 10 );
			}
			add_filter( 'rest_url_prefix', array( $this, 'rest_url_prefix' ) );
		}

		$login_captcha = isset( $this->options['login_captcha'] ) ? $this->options['login_captcha'] : '';
		$comment_captcha = isset( $this->options['comment_captcha'] ) ? $this->options['comment_captcha'] : '';

		if ( $login_captcha != '' ) {
			add_filter( 'login_form', array( $this, 'login_form' ) );
			add_filter( 'wp_authenticate_user', array( $this, 'authenticate_user_by_captcha' ), 9, 1 );

			add_action ( 'woocommerce_login_form', array( $this, 'login_form' ) );
		}

		if ( $comment_captcha != '' ) {
			if ( ! is_user_logged_in() ) {
				add_action( 'comment_form_after_fields', array( $this,  'comment_form_captcha_field' ), 1 );
				add_action( 'comment_form_logged_in_after', array( $this,  'comment_form_captcha_field' ), 1 );
				add_action( 'comment_form', array( $this,  'comment_form_captcha' ) );
				add_filter( 'preprocess_comment', array( $this, 'preprocess_comment_captcha' ) );
			}
		}

		if ( isset( $this->options['comment_bot'] ) && $this->options['comment_bot'] ) {
			if ( ! is_user_logged_in() ) {
				add_action( 'comment_form_after_fields', array( $this,  'comment_form_bot_field' ), PHP_INT_MAX );
				add_action( 'comment_form_logged_in_after', array( $this,  'comment_form_bot_field' ), PHP_INT_MAX );
				add_action( 'comment_form', array( $this,  'comment_form_bot' ) );
				add_filter( 'preprocess_comment' , array( $this, 'preprocess_comment_bot' ) );
			}
		}

		if (
			( isset( $this->options['comment_spam'] ) && $this->options['comment_spam'] )
			|| ( isset( $this->options['comment_spam_email'] ) && $this->options['comment_spam_email'] )
		) {
			add_filter( 'comment_form_field_comment' , array( $this, 'comment_form_field_comment_spam' ) );
			add_filter( 'preprocess_comment' , array( $this, 'preprocess_comment_spam' ) );
			add_filter( 'pre_comment_approved' , array( $this, 'pre_comment_approved' ), 99999, 2 );
		}

		if ( isset( $this->options['login_id_type'] ) ) {
			if ( $this->options['login_id_type'] == 'username' ) {
				remove_filter( 'authenticate', 'wp_authenticate_email_password', 20 );
				add_filter( 'login_form_defaults', array( $this, 'login_form_defaults' ) );
				add_filter( 'gettext', array( $this, 'gettext_login_id_username' ), 20, 3 );
			} if ( $this->options['login_id_type'] == 'email' ) {
				add_filter( 'authenticate', array( $this, 'authenticate_email' ), 20, 3 );
				add_filter( 'gettext', array( $this, 'gettext_login_id_email' ), 20, 3 );
			}
		}

		if ( isset( $this->options['password_reset'] ) && ! $this->options['password_reset'] ) {
			add_filter( 'gettext', array( $this, 'gettext_password_reset'), 10, 3 );
		}

		if ( isset( $this->options['login_site_link'] ) && ! $this->options['login_site_link'] ) {
			if ( version_compare( $wp_version, '5.7' ) >= 0 ) {
				add_filter( 'login_site_html_link', array( $this, 'login_site_html_link') );
			} else {
				add_filter( 'gettext_with_context', array( $this, 'gettext_login_site_link'), 10, 4 );
			}
		}

		if ( isset( $this->options['remove_version'] ) && $this->options['remove_version'] ) {
			remove_action( 'wp_head', 'wp_generator' );

			$this->hash_version = hash_hmac( 'md5', $wp_version, wp_salt( 'nonce' ) );
			add_filter( 'style_loader_src', array( $this, 'remove_meta_tag_version' ) );
			add_filter( 'script_loader_src', array( $this, 'remove_meta_tag_version' ) );
		}

		if ( isset( $this->options['edit_author_slug'] ) && $this->options['edit_author_slug'] ) {
			if ( current_user_can( 'administrator' ) ) {
				add_action( 'show_user_profile', array( $this, 'edit_user_profile' ) );
				add_action( 'edit_user_profile', array( $this, 'edit_user_profile' ) );
				add_action( 'user_profile_update_errors', array( $this, 'user_profile_update_errors' ), 20, 3 );
			}
		}

		if ( isset( $this->options['disable_feed'] ) && $this->options['disable_feed'] ) {
			add_action( 'init', array( $this, 'disable_feed' ) );
		}
	}

	private function get_login_type_value() {
		$type = ( basename( $_SERVER['SCRIPT_NAME'] ) == 'xmlrpc.php' ) ? self::LOGIN_TYPE_XMLRPC : self::LOGIN_TYPE_LOGIN_PAGE;
		return $type;
	}

	private function session_start() {
		if ( session_id() == '' ) {
			if ( ! defined( 'XMLRPC_REQUEST' ) ) {
				@ini_set( 'session.cookie_httponly', 1 );
				session_cache_limiter( 'private_no_expire' );
				session_start();
			}
		}
	}

	public function login_form_defaults( $defaults ) {
		$defaults['label_username'] = __( 'Username' );
		return $defaults;
	}

	public function gettext_login_id_username( $translated_text, $text, $domain ) {
		if ( 'wp-login.php' === $GLOBALS['pagenow'] ) {
			if ( $domain == 'default' && $text === 'Username or Email Address' ) {
				$translated_text = __( 'Username' );
			}
		}
		return $translated_text;
	}

	public function gettext_login_id_email( $translated_text, $text, $domain ) {
		if ( 'wp-login.php' === $GLOBALS['pagenow'] ) {
			if ( $domain == 'default' && $text === 'Username or Email Address' ) {
				$translated_text = __( 'Email' );
			}
		}
		return $translated_text;
	}

	public function rest_endpoints( $endpoints ) {
		if ( ! is_user_logged_in() ) {
			$s = isset( $this->options['rest_disable_endpoints'] ) ? $this->options['rest_disable_endpoints'] : '';
			if ( $s != '' ) {
				$disable_endpoints = explode( ',', $s );
				foreach ( $disable_endpoints as $disable_endpoint ) {
					foreach ( $endpoints as $key => $value ) {
						if ( $key == $disable_endpoint ) {
							unset( $endpoints[$key] );
						}
					}
				}
			}
		}
		return $endpoints;
	}

	public function disable_rest() {
		remove_action( 'xmlrpc_rsd_apis', 'rest_output_rsd' );
		if ( has_action( 'wp_head', 'rest_output_link_wp_head' ) ) {
			remove_action( 'wp_head', 'rest_output_link_wp_head', 10 );
		}
		remove_action( 'template_redirect', 'rest_output_link_header', 11, 0 );

		// REST API 1.x
		add_filter( 'json_enabled', '__return_false' );
		add_filter( 'json_jsonp_enabled', '__return_false' );

		// REST API 2.x
		add_filter( 'rest_enabled', '__return_false' );
		add_filter( 'rest_jsonp_enabled', '__return_false' );
	}

	public function rest_url_prefix() {
		if ( ! empty( $this->options['rest_name'] ) ) {
			return $this->options['rest_name'];
		}
	}

	public function authenticate( $user, $username, $password ) {
		if ( ! empty( $username ) ) {
			if ( null === $user || is_wp_error( $user ) ) {
				$this->failed_login( $username, $password );
			}
		}
		return $user;
	}

	public function handler_wp_login( $login, $current_user ) {
		if ( '' == $current_user->user_login ) {
			return;
		}
		$this->successful_login( $current_user, $this->get_login_type_value() );
	}

	public function handler_xmlrpc_call( $method ) {
		$current_user = wp_get_current_user();
		if ( '' == $current_user->user_login ) {
			return;
		}
		$this->successful_login( $current_user, self::LOGIN_TYPE_XMLRPC );
	}

	public function authenticate_user( $user, $password ) {
		if ( ! is_wp_error( $user ) && ! $this->is_login_ok() ) {
			$blocked_tarpit = isset( $this->options['blocked_tarpit'] ) ? (int) $this->options['blocked_tarpit'] : 0;
			if ( $blocked_tarpit > 0 ) {
				sleep( $blocked_tarpit );
			}
			$user = new WP_Error( 'limit_login', __( 'We restrict the login right now.', 'xo-security' ) );
		}
		return $user;
	}

	public function authenticate_user_by_captcha( $user ) {
		if ( ! is_wp_error( $user ) ) {
			if ( ! empty( $_SESSION['xo_security_captcha_login'] ) ) {
				if ( ! empty( $_POST['xo_security_captcha'] ) ) {
					$captcha = $_SESSION['xo_security_captcha_login'];
					//$_SESSION['xo_security_captcha_login'] = '';
					$input = $_POST['xo_security_captcha'];
					if ( $captcha != $input ) {
						$user = new WP_Error( 'captcha_login', __( '<strong>Error</strong>: The CAPTCHA code is wrong.', 'xo-security' ) );
					}
				} else {
					$user = new WP_Error( 'captcha_login', __( '<strong>Error</strong>: Please enter the CAPTCHA code.', 'xo-security' ) );
				}
			} else {
				if ( isset( $this->options['login_captcha'] ) && ! empty( $this->options['login_captcha'] ) ) {
					$user = new WP_Error( 'captcha_login', __( '<strong>Error</strong>: The session is unavailable. ', 'xo-security' ) );
				}
			}
		}
		return $user;
	}

	public function authenticate_email( $user, $username, $password ) {
		if ( null !== $user && ! is_wp_error( $user ) && $user->user_email != $username ) {
			$user = new WP_Error( 'invalid_username', __( '<strong>Error</strong>: There is no account with that email address.', 'xo-security' ) );
		}
		return $user;
	}

	public function login_error_message( $error ) {
		global $errors;

		if ( ! $this->is_login_ok() ) {
			$blocked_tarpit = isset( $this->options['blocked_tarpit'] ) ? (int) $this->options['blocked_tarpit'] : 0;
			if ( $blocked_tarpit > 0 ) {
				sleep( $blocked_tarpit );
			}
			return __( '<strong>Error</strong>: We restrict the login right now.', 'xo-security' );
		}

		if ( is_wp_error( $errors ) ) {
			$err_codes = $errors->get_error_codes();
			if ( in_array( 'captcha_login', $err_codes ) ) {
				return $error;
			}
		}

		if ( isset( $this->options['error_login_message'] ) && $this->options['error_login_message'] ) {
			return __( '<strong>Error</strong>: Username or password is incorrect.', 'xo-security' );
		}

		return $error;
	}

	public function login_init() {
		if ( isset( $_GET['action'] ) && in_array( $_GET['action'], array( 'postpass', 'confirmaction', 'logout' ) ) ) {
			return;
		}

		if ( isset( $_GET['checkemail'] ) && $_GET['checkemail'] === 'confirm' ) {
			return;
		}

		if ( is_user_logged_in() ) {
			return;
		}

		if ( defined( 'XO_SECURITY_LANGUAGE_WHITE_LIST' ) && XO_SECURITY_LANGUAGE_WHITE_LIST !== '' ) {
			$lang = strtolower( substr( $this->get_language(), 0, 2 ) );
			$whitelangs = explode( ',', strtolower( XO_SECURITY_LANGUAGE_WHITE_LIST ) );
			if( ! in_array( $lang, $whitelangs ) ) {
				$blocked_tarpit = isset( $this->options['blocked_tarpit'] ) ? (int) $this->options['blocked_tarpit'] : 0;
				if ( $blocked_tarpit > 0 ) {
					sleep( $blocked_tarpit );
				}
				$this->redirect_404();
				exit;
			}
		}

		if ( ! $this->is_login_ok() ) {
			$this->redirect_404();
			exit;
		}

		if ( isset( $this->options['login_page'] ) && $this->options['login_page'] ) {
			$login_page_name = ( isset( $this->options['login_page_name'] ) ? $this->options['login_page_name'] : '' );
			if ( strpos( $_SERVER['REQUEST_URI'], $login_page_name ) === false ) {
				$this->redirect_404();
				exit;
			}
		}
	}

	private function redirect_404( $redirect = true ) {
		if ( $redirect ) {
			wp_safe_redirect( home_url( '/404' ) );
		} else {
			global $wp_query;

			status_header( 404 );
			if ( function_exists( 'nocache_headers' ) ) {
				nocache_headers();
			}
			$wp_query->set_404();
			$page_404 = get_404_template();
			if ( 1 < strlen( $page_404 ) ) {
				include( $page_404 );
			} else {
				include( get_query_template( 'index' ) );
			}
		}
	}

	private function get_login_url( $url, $loginfile ) {
		if ( is_user_logged_in() || strpos( $_SERVER['REQUEST_URI'], $loginfile ) !== false ) {
			$url = str_replace( 'wp-login.php', $loginfile, $url );
		} elseif ( strpos( $_SERVER['REQUEST_URI'],  '/wp-activate.php' ) !== false ) {
			if ( ! empty( $_GET['key'] ) ) {
				$key = $_GET['key'];
			} elseif ( ! empty( $_POST['key'] ) ) {
				$key = $_POST['key'];
			}
			if ( $key ) {
				$result = wpmu_activate_signup( $key );
			}
			if ( null === $result && isset( $_COOKIE['wp-activate-' . COOKIEHASH] ) ) {
				$key = $_COOKIE[$activate_cookie];
				$result = wpmu_activate_signup( $key );
			}
			if ( null !== $result ) {
				if ( is_wp_error( $result ) ) {
					$error_code = $result->get_error_code();
				}
				if ( isset( $error_code ) || $error_code === 'user_already_exists' || $error_code === 'already_active' ) {
					$url = str_replace( 'wp-login.php', $loginfile, $url );
				}
			}
		}
		return $url;
	}

	public function site_url( $url, $path, $scheme, $blog_id ) {
		if ( isset( $this->options['login_page_name'] ) ) {
			$loginfile = $this->options['login_page_name'] . '.php';
			$formated_path = '/' . ltrim( $path, '/' );
			if ( $formated_path === '/wp-login.php' || preg_match( '#/wp-login\.php\?action=\w+#', $formated_path ) ) {
				$url = $this->get_login_url( $url, $loginfile );
			} elseif ( is_multisite() && ! is_subdomain_install() ) {
				$blog = get_site( $blog_id );
				if (  $formated_path === $blog->path . 'wp-login.php' || preg_match( '#' . $blog->path . 'wp-login\.php\?action=\w+#', $formated_path ) ) {
					$url = $this->get_login_url( $url, $loginfile );
				}
			}
		}
		return $url;
	}

	public function network_site_url( $url, $path, $scheme ) {
		return $this->site_url( $url, $path, $scheme, get_current_blog_id() );
	}

	public function login_page_wp_redirect( $location, $status ) {
		if ( isset( $this->options['login_page_name'] ) ) {
			$loginfile = $this->options['login_page_name'] . '.php';
			if ( strpos( $_SERVER['REQUEST_URI'], $loginfile ) !== false ) {
				$location = str_replace( 'wp-login.php', $loginfile, $location );
			}
		}
		return $location;
	}

	public function recreate_login_file() {
		global $wp, $wp_filesystem;

		$login_page_filename = ( isset( $this->options['login_page_name'] ) ? $this->options['login_page_name'] . '.php' : '' );
		if ( $wp->request == $login_page_filename ) {
			$path = ABSPATH . $login_page_filename;
			if ( ! file_exists( $path ) ) {
				require_once ABSPATH . 'wp-admin/includes/file.php';
				if ( WP_Filesystem() ) {
					$loginfile_content = "<?php require_once './wp-login.php'; ?>";
					if ( $wp_filesystem->put_contents( $path, stripslashes( $loginfile_content ), FS_CHMOD_FILE ) ) {
						wp_safe_redirect( home_url( $login_page_filename ) );
						exit();
					}
				}
			}
		}
	}

	public function xmlrpc_enabled() {
		$blocked_tarpit = isset( $this->options['blocked_tarpit'] ) ? (int)$this->options['blocked_tarpit'] : 0;
		if ( $blocked_tarpit > 0 ) {
			sleep( $blocked_tarpit );
		}
		return false;
	}

	public function xmlrpc_login_error_message( $error ) {
		if ( isset( $this->options['error_login_message'] ) && $this->options['error_login_message'] ) {
			return new IXR_Error( 403, __( 'Incorrect username or password.', 'xo-security' ) );
		}
		return $error;
	}

	public function login_failure_shake( $error_codes ) {
		$error_codes[] = 'limit_login';
		return $error_codes;
	}

	public function author_rewrite_rules() {
		return array();
	}

	public function remove_author_sitemap_provider( $provider, $name ) {
		if ( 'users' === $name ) {
			return false;
		}
		return $provider;
	}

	public function author_rewrite() {
		if ( is_admin() ) {
			return;
		}

		$author = filter_input( INPUT_GET, 'author' );
		$uri = @$_SERVER['REQUEST_URI'];
		if ( $author || preg_match( '#/author/.+#', $uri ) ) {
			$this->redirect_404();
			exit;
		}
	}

	public function remove_comment_author_class( $classes ) {
		foreach ( $classes as $key => $class ) {
			if ( strstr( $class, 'comment-author-' ) ) {
				unset( $classes[$key] );
			}
		}
		return $classes;
	}

	public function filter_oembed_response_data( $data ) {
		unset( $data['author_url'] );
		unset( $data['author_name'] );
		// $data['author_name'] = get_bloginfo( 'name' );
		// $data['author_url']  = get_home_url();
		return $data;
	}

	public function remove_pingback( $methods ) {
		unset( $methods['pingback.ping'] );
		unset( $methods['pingback.extensions.getPingbacks'] );
		return $methods;
	}

	public function remove_x_pingback( $headers ) {
		unset( $headers['X-Pingback'] );
		return $headers;
	}

	/**
	 * Get the IP address of the client.
	 *
	 * @since 2.4.0
	 * 
	 * @return string|false The valid IP address, otherwise false.
	 */
	public function get_auto_ipaddress() {
		$ip = false;
		if ( ! empty( $_SERVER['HTTP_X_REAL_IP'] ) ) {
			$ip = filter_var( $_SERVER['HTTP_X_REAL_IP'], FILTER_VALIDATE_IP );
		} else if ( ! empty( $_SERVER['REMOTE_ADDR'] ) ) {
			$ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP );
		} else if ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
			$ips = explode( ',', $_SERVER['HTTP_X_FORWARDED_FOR'] );
			$ip = filter_var( trim( $ips[0] ), FILTER_VALIDATE_IP );
		} else if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
			$ip = filter_var( $_SERVER['HTTP_CLIENT_IP'], FILTER_VALIDATE_IP );
		}
		return $ip;
	}

	/**
	 * Get the IP address of the client.
	 * 
	 * @since 1.0.0
	 * 
	 * @return string The valid IP address, otherwise ''.
	 */
	private function get_ipaddress() {
		$mode = isset( $this->options['ip_mode'] ) ? $this->options['ip_mode'] : '';

		$ip = false;
		if ( $mode == 'remote_addr' ) {
			if ( ! empty( $_SERVER['REMOTE_ADDR'] ) ) {
				$ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP );
			}
		} else if ( $mode == 'http_x_forward_for_1' || $mode == 'http_x_forward_for_2' || $mode == 'http_x_forward_for_3' ) {
			if ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
				$ips = explode( ',', $_SERVER['HTTP_X_FORWARDED_FOR'] );
				$ips = array_pad( $ips, 3, '' );
				switch ( $mode ) {
					case 'http_x_forward_for_2':
						$ip = filter_var( trim( $ips[1] ), FILTER_VALIDATE_IP );
						break;
					case 'http_x_forward_for_3':
						$ip = filter_var( trim( $ips[2] ), FILTER_VALIDATE_IP );
						break;
					default:
						$ip = filter_var( trim( $ips[0] ), FILTER_VALIDATE_IP );
				}
			}
		} else if ( $mode == 'http_x_real_ip' ) {
			if ( ! empty( $_SERVER['HTTP_X_REAL_IP'] ) ) {
				$ip = filter_var( $_SERVER['HTTP_X_REAL_IP'], FILTER_VALIDATE_IP );
			}
		} else if ( $mode == 'http_client_ip' ) {
			if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
				$ip = filter_var( $_SERVER['HTTP_CLIENT_IP'], FILTER_VALIDATE_IP );
			}
		} else {
			$ip = $this->get_auto_ipaddress();
		}

		if ( $ip ) {
			return $ip;
		}
		return '';
	}

	/**
	 * Get USER AGENT.
	 */
	private function get_user_agent() {
		$user_agent = isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : '';
		return substr( sanitize_text_field( $user_agent ), 0, 254 );
	}

	/**
	 * Get the browser's main language.
	 */
	public function get_language() {
		$result = null;
		if ( isset ( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ) {
			$langs = explode( ',', sanitize_text_field( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) );
			if ( count( $langs ) >= 1 ) {
				$main_lang = explode( ';', $langs[0] );
				if ( count( $main_lang ) >= 1 ) {
					$result = substr( trim( $main_lang[0] ), 0, 5 );
				}
			}
		}
		return $result;
	}

	/**
	 * Record login failures.
	 *
	 * @since 1.7.0
	 *
	 * @param string $username Username for authentication.
	 * @param string $password Optional. Password for authentication.
	 */
	private function failed_login( $username, $password = null ) {
		global $wpdb;

		if ( empty( $username ) ) {
			return;
		}

		$login_time = current_time( 'mysql' );
		$ip_address = $this->get_ipaddress();
		$type = $this->get_login_type_value();
		$lang = $this->get_language();
		$user_agent = $this->get_user_agent();
		// if ( null === $password ) { $password = sanitize_text_field( filter_input( INPUT_POST, 'pwd' ) ); }
		if ( null !== $password && get_user_by( 'login', $username ) ) {
			$password = null;
		}

		$table = $wpdb->prefix . 'xo_security_loginlog';

		$wpdb->insert( $table, array(
			'success'         => false,
			'login_time'      => $login_time,
			'ip_address'      => $ip_address,
			'login_type'      => $type,
			'lang'            => $lang,
			'user_agent'      => $user_agent,
			'user_name'       => $username,
			'failed_password' => $password,
		) );

		$failed_tarpit = isset( $this->options['failed_tarpit'] ) ? (int) $this->options['failed_tarpit'] : 0;
		if ( $failed_tarpit > 0 ) {
			sleep( $failed_tarpit );
		}
	}

	/**
	 * Record login successful.
	 *
	 * @since 1.7.0
	 *
	 * @param string $user WP_User Current WP_User instance.
	 * @param string $type Login type.
	 */
	private function successful_login( $user, $type ) {
		global $wpdb;

		$user_name = $user->user_login;
		$login_time = current_time( 'mysql' );
		$ip_address = $this->get_ipaddress();
		$lang = $this->get_language();
		$user_agent = $this->get_user_agent();

		$table = $wpdb->prefix . 'xo_security_loginlog';

		$wpdb->insert( $table, array(
			'success'         => true,
			'login_time'      => $login_time,
			'ip_address'      => $ip_address,
			'login_type'      => $type,
			'lang'            => $lang,
			'user_agent'      => $user_agent,
			'user_name'       => $user_name,
			'failed_password' => null,
		) );

		// Automatic deletion of old login log.
		$truncate_date = isset( $this->options['auto_truncate'] ) ? intval( $this->options['auto_truncate'] ) : 0;
		if ( $truncate_date > 0 ) {
			$wpdb->query( $wpdb->prepare( "DELETE FROM {$table} WHERE `login_time` <= DATE_SUB(NOW(), INTERVAL %d day)", $truncate_date ) );
		}

		$count = $wpdb->get_var( "SELECT COUNT(*) FROM {$table}" );
		if ( $count > self::MAX_LOGIN_LOG_RECORDS ) {
			$wpdb->query( $wpdb->prepare( "DELETE FROM {$table} ORDER BY `id` ASC LIMIT %d", ( $count - self::MAX_LOGIN_LOG_RECORDS ) ) );
		}

		$login_alert = isset( $this->options['login_alert'] ) ? $this->options['login_alert'] : false;
		if ( $login_alert ) {
			$login_alert_admin_only = isset( $this->options['login_alert_admin_only'] ) ? $this->options['login_alert_admin_only'] : false;
			if ( $login_alert_admin_only ) {
				if ( $user->has_cap( 'administrator' ) ) {
					$this->send_login_mail( $user, $ip_address, $user_agent );
				}
			} else {
				$this->send_login_mail( $user, $ip_address, $user_agent );
			}
		}
	}

	private function send_login_mail( $user, $ip_address, $user_agent ) {
		$user_email = $user->user_email;

		$user_name = $user->user_login;
		$site_name = get_bloginfo( 'name' );
		$subject = isset( $this->options['login_alert_subject'] ) ? $this->options['login_alert_subject'] : '';
		$body = isset( $this->options['login_alert_body'] ) ? $this->options['login_alert_body'] : '';

		$subject = str_replace( array( '%SITENAME%', '%USERNAME%', '%DATE%', '%TIME%', '%IPADDRESS%', '%USERAGENT%' ),
					array( $site_name, $user_name, date_i18n( 'Y-m-d' ), date_i18n( 'H:i:s' ), $ip_address, $user_agent ), $subject );
		$body = str_replace( array( '%SITENAME%', '%USERNAME%', '%DATE%', '%TIME%', '%IPADDRESS%', '%USERAGENT%' ),
					array( $site_name, $user_name, date_i18n( 'Y-m-d' ), date_i18n( 'H:i:s' ), $ip_address, $user_agent ), $body );

		wp_mail( $user_email, esc_html( $subject ), esc_html( $body ) );
	}

	private function is_login_ok() {
		global $wpdb;

		$locked = false;
		$ipaddress_esc = esc_sql( $this->get_ipaddress() );
		$interval_hour = isset( $this->options['interval'] ) ? (int) $this->options['interval'] : 0;
		if ( $interval_hour > 0 ) {
			$table = $wpdb->prefix . 'xo_security_loginlog';
			$time = date( 'Y-m-d H:i:s', (int) current_time( 'timestamp' ) - ( $interval_hour * 60 * 60 ) );
			$success_login_time = $wpdb->get_var( "SELECT login_time FROM {$table} WHERE ip_address='{$ipaddress_esc}' AND success=1 ORDER BY login_time DESC LIMIT 1" );
			if ( $success_login_time !== null ) {
				$time = max( $time, $success_login_time );
			}
			$count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$table} WHERE ip_address='{$ipaddress_esc}' AND success=0 AND login_time > %s;", $time ) );
			$limit_count = isset( $this->options['limit_count'] ) ? (int) $this->options['limit_count'] : 1;
			if ( $count >= $limit_count ) {
				$locked = true;
			}
		}

		if ( ! $locked ) {
			$user_agent = $this->get_user_agent();
			if ( $user_agent === '' ) {
				$locked = true;
			} else {
				if ( defined( 'XO_SECURITY_UA_WHITE_LIST' ) && XO_SECURITY_UA_WHITE_LIST !== '' ) {
					$locked = true;
					$whites = explode( ',', XO_SECURITY_UA_WHITE_LIST );
					foreach ( $whites as $white ) {
						if ( ! ( stripos( $user_agent, $white ) === false ) ) {
							$locked = false;
							break;
						}
					}
				}
				if ( ! $locked ) {
					if ( defined( 'XO_SECURITY_UA_BLACK_LIST' ) && XO_SECURITY_UA_BLACK_LIST !== '' ) {
						$blocks = explode( ',', XO_SECURITY_UA_BLACK_LIST );
						foreach ( $blocks as $block ) {
							if ( ! ( stripos( $user_agent, $block ) === false ) ) {
								$locked = true;
								break;
							}
						}
					}
				}
			}
		}

		return ! $locked;
	}

	public function login_form() {
		$char_mode = isset( $this->options['login_captcha'] ) ? $this->options['login_captcha'] : 'en';
		$src = XO_SECURITY_URL . '/captcha/captcha.php?prefix=login&char_mode=' . urlencode( $char_mode );
		echo '<p><img id="xo-security-captcha" src="' . esc_url( $src ) . '" alt="CAPTCHA" width="100" height="36"></p>';
		echo '<p>';
		echo '<label for="xo_security_captcha">' . esc_html__( 'CAPTCHA Code', 'xo-security' ) . '</label><br />';
		echo '<input type="text" name="xo_security_captcha" id="xo_security_captcha" class="input" value="" size="10" aria-required="true" autocomplete="off" />';
		echo '</p>' . "\n";
	}

	public function display_comment_form_captcah() {
		$char_mode = isset( $this->options['comment_captcha'] ) ? $this->options['comment_captcha'] : 'en';
		$src = XO_SECURITY_URL . '/captcha/captcha.php?prefix=comment&char_mode=' . urlencode( $char_mode );
		echo '<p><img id="xo-security-captcha" src="' . esc_url( $src ) . '" alt="CAPTCHA" width="100" height="36" loading="lazy"></p>';
		echo '<p class="comment-form-captcha">';
		echo '<label for="xo_security_captcha">' . esc_html__( 'CAPTCHA Code', 'xo-security' ) . '</label>';
		echo '<input type="text" name="xo_security_captcha" id="xo_security_captcha" value="" size="10" aria-required="true" autocomplete="off" />';
		echo '</p>' . "\n";
	}

	public function comment_form_captcha_field() {
		remove_action( 'comment_form', array( $this, 'comment_form_captcha' ) );
		$this->display_comment_form_captcah();
	}

	public function comment_form_captcha() {
		$this->display_comment_form_captcah();
	}

	public function preprocess_comment_captcha( $comment ) {
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'replyto-comment' &&
			( check_ajax_referer( 'replyto-comment', '_ajax_nonce', false ) || check_ajax_referer( 'replyto-comment', '_ajax_nonce-replyto-comment', false ) )
		) {
			return $comment;
		}
		if ( $comment['comment_type'] != '' && $comment['comment_type'] != 'comment' ) {
			return $comment;
		}
		if ( ! empty( $_SESSION['xo_security_captcha_comment'] ) && ! empty( $_POST['xo_security_captcha'] ) ) {
			$captcha = $_SESSION['xo_security_captcha_comment'];
			$input = $_POST['xo_security_captcha'];
			if ( $captcha == $input ) {
				//$_SESSION['xo_security_captcha_comment'] = '';
				return $comment;
			}
		}
		wp_die( __( '<strong>Error</strong>: Invalid CAPTCHA code.', 'xo-security' ), __( 'Error', 'xo-security' ), array( 'back_link' => true ) );
	}

	public function display_comment_form_bot_field() {
		echo '<p class="comment-form-bot-check">';
		echo '<input id="comment-form-bot-check" name="comment-form-bot-check" type="checkbox" value="yes">';
		echo '<label for="comment-form-bot-check" class="comment-notes" style="display: inline;">' . esc_html__( "I'm not a robot.", 'xo-security' ) .'</label>';
		echo '</p>';
		echo '<script async defer>var spamtimeout=function(){ document.getElementById("comment-form-bot-check").value = "OK"; };setTimeout(spamtimeout, 1000);</script>';
	}

	public function comment_form_bot_field() {
		remove_action( 'comment_form', array( $this, 'comment_form_bot' ) );
		$this->display_comment_form_bot_field();
	}

	public function comment_form_bot() {
		$this->display_comment_form_bot_field();
	}

	public function preprocess_comment_bot( $comment ) {
		$check = isset( $_POST['comment-form-bot-check'] ) ? sanitize_text_field( $_POST['comment-form-bot-check'] ) : '';
		if ( '' === $check ) {
			wp_die( __( '<strong>Error</strong>: Please check the anti-bot checkbox.', 'xo-security' ), __( 'Error', 'xo-security' ), array( 'back_link' => true ) );
		} else if ( 'OK' !== $check ) {
			wp_die( __( '<strong>Error</strong>: Please enable JavaScript in your browser.', 'xo-security' ), __( 'Error', 'xo-security' ), array( 'back_link' => true ) );
		}
		return $comment;
	}

	public function comment_form_field_comment_spam( $field ) {
		if (
			( isset( $this->options['comment_spam'] ) && $this->options['comment_spam'] )
			&& ( isset( $this->options['comment_spam_message'] ) && $this->options['comment_spam_message'] )
		) {
			$field .= '<p class="comment-comment-message">' . esc_html( $this->options['comment_spam_message'] ) . '</p>';
		}
		return $field;
	}

	public function preprocess_comment_spam( $comment ) {
		$is_spam = false;

		if ( isset( $this->options['comment_spam'] ) && $this->options['comment_spam'] ) {
			$comment_content = stripslashes( $comment["comment_content"] );
			if ( ! preg_match( '/[一-龠]+|[ぁ-ん]+|[ァ-ヴー]/u', $comment_content ) ) {
				$is_spam = true;
			}
		}

		if ( isset( $this->options['comment_spam_email'] ) && $this->options['comment_spam_email'] ) {
			$spam = get_comments( array(
					'status'       => 'spam',
					'number'       => 1,
					'author_email' => $comment['comment_author_email'],
					'count'        => true,
			) );
			if ( $spam > 0 ) {
				$is_spam = true;
			}
		}

		if ( $is_spam ) {
			$action = isset( $this->options['comment_spam_action'] ) ? $this->options['comment_spam_action'] : 'block';
			if ( 'spam' === $action ) {
				$comment['spam'] = true;
			} else if ( 'trash' === $action ) {
				$comment['trash'] = true;
			} else {
				wp_die( __( '<strong>Error</strong>: Invalid Comment.', 'xo-security' ), __( 'Error', 'xo-security' ), array( 'back_link' => true ) );
			}
		}

		return $comment;
	}

	public function pre_comment_approved( $approved, $commentdata ) {
		if ( isset( $commentdata['spam'] ) && $commentdata['spam'] ) {
			$approved = 'spam';
		} else if ( isset( $commentdata['trash'] ) && $commentdata['trash'] ) {
			$approved = 'trash';
		}
		return $approved;
	}

	public function gettext_password_reset( $translation, $text, $domain ) {
		if ( 'default' == $domain && 'Lost your password?' == $text ) {
			$translation = '';
		}
		return $translation;
	}

	public function gettext_login_site_link( $translation, $text, $context, $domain ) {
		if ( 'default' == $domain && 'site' == $context ) {
			// WP 4.6-5.5: '&larr; Back to %s'
			// WP 5.6:     '&larr; Go to %s'
			if ( '&larr; Back to %s' == $text || '&larr; Go to %s' == $text ) {
				$translation = '';
			}
		}
		return $translation;
	}

	public function login_site_html_link( $link ) {
		return '';
	}

	public function remove_meta_tag_version( $src ) {
		global $wp_version;

		if ( false !== strpos( $src, '?ver=' . $wp_version ) ) {
			$src = add_query_arg( 'ver', $this->hash_version, $src );
		}
		return $src;
	}

	/**
	 * Display 'Nicename' item on the 'Edit User' screen.
	 *
	 * @since 3.0.0
	 *
	 * @param object $profileuser A WP_User object
	 */
	public function edit_user_profile( $profileuser ) {
		?>
		<h2><?php esc_html_e( 'Slug', 'xo-security' ); ?></h2>
		<table class="form-table" role="presentation">
		<tbody>
		<tr class="user-last-name-wrap">
			<th><label for="user_nicename"><?php esc_html_e( 'Author Slug (Nicename)', 'xo-security' ); ?></label></th>
			<td>
			<input type="text" name="user_nicename" id="user_nicename" value="<?php echo esc_attr( $profileuser->user_nicename ); ?>" class="regular-text" />
			<span class="description"><?php esc_html_e( 'If omitted, it will be the username.', 'xo-security' ); ?></span>
			<p class="description"><?php esc_html_e( 'Used for things like the URL of an author archive page.', 'xo-security' ); ?></p>
			</td>
		</tr>
		</tbody></table>
		<?php
	}

	/**
	 * Fires before user profile update errors are returned.
	 *
	 * @since 3.0.0
	 */
	public function user_profile_update_errors( $errors, $update, $user ) {
		if ( ! $update ) {
			return;
		}

		if ( empty( $user->ID ) ) {
			return;
		}

		check_admin_referer( 'update-user_' . $user->ID );

		if ( isset( $_POST['user_nicename'] ) ) {
			$user_nicename = sanitize_user( wp_unslash( $_POST['user_nicename'] ), true );

			$old_user_nicename = get_user_by( 'id', $user->ID )->user_nicename;
			if ( $user_nicename === $old_user_nicename ) {
				return;
			}

			if ( mb_strlen( $user_nicename ) > 50 ) {
				$errors->add( 'user_nicename_too_long', __( '<strong>Error</strong>: The author slug may not be longer than 50 characters.', 'xo-security' ) );
				return;
			}

			if ( get_user_by( 'slug', $user_nicename ) ) {
				$errors->add( 'user_nicename_exists', __( '<strong>Error</strong>: The author slug with the same name already exists.', 'xo-security' ) );
				return;
			}

			$user->user_nicename = $user_nicename;
		}
	}

	/**
	 * Disable RSS and Atom feeds.
	 *
	 * @since 3.1.0
	 */
	public function disable_feed() {
		remove_action( 'do_feed_rdf', 'do_feed_rdf', 10, 0 );
		remove_action( 'do_feed_rss', 'do_feed_rss', 10, 0 );
		remove_action( 'do_feed_rss2', 'do_feed_rss2', 10, 1 );
		remove_action( 'do_feed_atom', 'do_feed_atom', 10, 1 );

		remove_action( 'wp_head', 'feed_links', 2 );
		remove_action( 'wp_head', 'feed_links_extra', 3 );
	}
}
